/**
 * Clase que permite la creacion de un robot con sensores.
 * Implementa un robot movible con un vector velocidad, posicion, sensores de
 * distancia, un mundo en el que se situa y una serie de variables internas
 * necesarias para su funcionamiento.
 *
 * Autor: Matias Contreras, Francisco Frez, Damian Quiroz.
 * Version: 1.0
 * Since: 2018-07-21
*/

#ifndef ROBOT_h				// Comprueba si la etiqueta no esta definida
#define ROBOT_h				// Crea una bandera en tiempo de compilacion

#include "Vector2D.h"			// Biblioteca de cabecera que permite usar vectores
#include "DistanceSensor.h"		// Biblioteca de cabecera que permite usar los sensores

class MyWorld;			// Instancia la clase correspondiente al mundo simulado

class Robot {
	public:
	/* Constructores y metodos */
		Robot();
		Robot(Vector2D position, Vector2D velocity, double sensorRange, MyWorld* world);
		void turnLeft();
		void turnRight();
		void moveDeltaT(double deltaT);
		void moveRobot(double deltaT);
		void operator=(Robot robot);
		string getDescription() {return _position.getDescription();}
		string toString() {return _position.toString();}
		Vector2D getPosition() {return _position;}
		Vector2D getVelocity() {return _velocity;}

	private:
	/* Definicion de variables */
		Vector2D _position;
		Vector2D _velocity;
		MyWorld* _world;
		DistanceSensor *_rightSensor, *_frontSensor, *_leftSensor;
};

#endif 		/* Codigo se incluye en el paquete
	 	 	 * que se envia al compilador hasta aqui */
