/*
 * Implementa los constructores y metodos que permiten rotar el laberinto
 * (trabaja con archivos de extension "pbm").
 *
 * Autor: Matias Contreras, Francisco Frez, Damian Quiroz.
 * Version: 2.0
 * Since: 2018-07-29
 */

#include <string> 			// Biblioteca para utilizar funciones sobre strings
#include <limits>			// Biblioteca para funciones que definen limites
#include "Maze.h"			/* Incluye libreria de cabecera que define
							 * los constructores y metodos a implementar */
#include <stdlib.h>
using namespace std;		// Incluye espacio  de nombres para palabras reservadas

Maze::Maze(ifstream& data){
	_width = 0, _height = 0;
	string read, aux;
	data.ignore(numeric_limits<streamsize>::max(), '\n');
	data.ignore(numeric_limits<streamsize>::max(), '\n');
	data.ignore(numeric_limits<streamsize>::max(), '\n');

	getline(data, read, ' ');
	_width = atoi(read.c_str());		// Obtiene ancho desde archivo
	getline(data, read);
	_height = atoi(read.c_str());		// Obtiene largo desde archivo

	_array = new bool*[_height];					// Define alto del arreglo
	for (int h = 0; h < _height; h++) {
		_array[h] = new bool[_width];				// Define ancho del arreglo
	}

	getline(data, read);
	while (!data.eof()) {
		getline(data, aux);		// Lee linea por linea desde data
		read += aux;			// Las almacena en un string
	}

	for (int h = 0; h < _height; h++) {
		for (int w = 0; w < _width; w++) {			// Guarda informacion en el arreglo
			(read[(h*_width) + w] == '1') ?
				_array[h][w] = true : _array[h][w] = false;
		}
	}
	/* Constructor que se encarga de obtener y almacenar informacion desde data */
}

Maze::Maze(Maze& maze){
	_width = maze.getWidth();
	_height = maze.getHeight();

	_array = new bool*[_height];
	for (int h = 0; h < _height; h++) {
		_array[h] = new bool[_width];
		for (int w = 0; w < _width; w++) {
			_array[h][w] = (maze.getArray())[h][w];
		}
	}
	/* Metodo que se encarga de asignar el laberinto a la variable _array */
}

void Maze::rotate(){
	int new_width = _height;		// Cambio de alto por ancho
	int new_height = _width;		// Cambio de ancho por alto

	/* Creacion de la matriz rotada */
	bool** rotated;
	rotated = new bool*[_width];
	for (int h = 0; h < _width; h++) {
		rotated[h] = new bool[_height];
	}

	/* Se asignan los valores a la matriz rotada */
	for (int h = 0; h < _height; h++) {
		for (int w = 0; w < _width; w++) {
			rotated[_width - 1 - w][h] = _array[h][w];
		}
	}

	/* Se limpia la memoria usada por _array */
	 for (int h = 0; h < _height; ++h)
		 delete [] _array[h];
	 delete [] _array;

	_array = rotated;				// Se asignan las variables internas de la clase
	_width = new_width;
	_height = new_height;
	/* Metodo que se encarga de rotar el laberinto entregado en data */
}

void Maze::write(ofstream& rotate){
	rotate << "P1" << endl;
	rotate << "# Created by Matias - Francisco - Damian UTFSM ELO329" << endl;	// Mensajes inicio del archivo
	rotate << _width << " " << _height << endl;

	/* Escribe en rotate los valores desde la matriz array */
	for(int i = 0; i < _height; i++){
		string lab = " ";
		for(int j = 0; j < _width; j++){
			if(_array[i][j] == true) lab += "1";
			else lab += "0";
		}
		rotate << lab << endl;
	}
	/* Metodo que escribe el archivo nuevo con el laberinto rotado */
}

bool Maze::isThere_a_wall(int x, int y) {
	return (x >= _width || y >= _height || y <= 0 || x <= 0 || _array[y][x]);
	/* Metodo que retorna true en caso de hallar el robot una muralla */
}

void Maze::traceRoute(Vector2D position) {
	_array[(int) position.getY()][(int) position.getX()] = 1;
	/* Metodo que se encarga de trazar la ruta del robot en el laberinto */
}

