/*
 * Clase principal del programa.
 * Implementa los constructores y metodos esenciales que permiten el inicio del
 * programa, asi como la interaccion entre todas las clases.
 *
 * Autor: Matias Contreras, Francisco Frez, Damian Quiroz.
 * Version: 1.0
 * Since: 2018-07-25
 */

#include <iostream>				// Biblioteca para las operaciones de entrada y salida
#include <fstream>				// Biblioteca para el tratamiendo de archivos
#include <string> 				// Biblioteca para utilizar funciones sobre strings
#include <limits>				// Biblioteca para funciones que definen limites
#include "MyWorld.h"			// Biblioteca de cabecera que define el mundo a utilizar
#include "Robot.h"				// Biblioteca de cabecera que permite usar robots
#include "Vector2D.h"			// Biblioteca de cabecera que permite usar vectores
#include "Maze.h"				// Biblioteca de cabecera para el uso del laberinto
#include "DocumentedMaze.h"		// Biblioteca de cabecera para obtener informaci�n del laberinto
using namespace std;			// Incluye espacio de nombres para palabras reservadas

int main(int argc, char* argv[]) {
	ifstream maze_in;					// Laberinto original
	ofstream maze_out_1;				// Laberinto donde se escribe la ruta (pared a la derecha)
	ofstream maze_out_2;				// Laberinto donde se escribe la ruta (pared a la izquierda)

	if (argc < 3) {
		cout << "Usage: ./Stage5 <pbm input file> <pbm output file 1> <pbm output file 2> [x  y  dx  dy  fx  fy  r]" << endl;
		exit(1);
	}

	maze_in.open(argv[1]);		// maze_in.open("simple_maze.pbm");
	if (maze_in.fail()) {
		cout << "Error al abrir el archivo 1" << argv[1] << endl;
		exit(1);
	}

	maze_out_1.open(argv[2]);		// maze_out.open("robot_route_1.pbm");
	if (maze_out_1.fail()) {
		cout << "Error al abrir el archivo 2" << argv[2] << endl;
		exit(1);
	}

	maze_out_2.open(argv[3]);		// maze_out.open("robot_route_2.pbm");
	if (maze_out_2.fail()) {
		cout << "Error al abrir el archivo 3" << argv[3] << endl;
		exit(1);
	}

	DocumentedMaze maze(maze_in);
	Maze route_1(maze);					// escribe la ruta pegado a la derecha
	Maze route_2(maze);					// escribe la ruta pegado a la izquierda

	if (argc > 5)
		maze.setEntrance(atof(argv[4]), atof(argv[5]));		// posicion de entrada (x,y)
	else if (argc > 7)
		maze.setDirection(atof(argv[6]), atof(argv[7]));	// direccion de entrada (dx,dy)
	else if (argc > 9)
		maze.setExit(atof(argv[8]), atof(argv[9]));			// salida del laberinto (fx,fy)
	else if (argc > 10)
		maze.setExitRadius(atof(argv[10]));					// radio de salida r

	double deltaT = 1.0; 				// [s]
	double speed = 1.0;					// [m/s]
	double sensorRange = 5; 			// [m]
	MyWorld* world = new MyWorld;		// crea mundo simulado
	Robot* robot_1;						// crea un robot
	Robot* robot_2;						// crea otro robot

	Vector2D position_1 = maze.getEntrance();							// vector de posicion inicial
	Vector2D velocity_1 = maze.getDirection().getUnitary() * speed;		// vector de velocidad inicial
	robot_1 = new Robot(position_1, velocity_1, sensorRange, world);		// otorga caracteristicas al robot

	Vector2D position_2 = maze.getEntrance();							// vector de posicion inicial
	Vector2D velocity_2 = maze.getDirection().getUnitary() * speed;		// vector de velocidad inicial
	robot_2 = new Robot(position_2, velocity_2, sensorRange, world);		// otorga caracteristicas al robot

	world->setMaze(maze);							// setea laberinto en el mundo
	world->setRobot(robot_1, robot_2);				// setea robot y otro robot en el mundo
	world->printStateDescription();					// imprime en consola el estado
	world->simulate(deltaT, route_1, route_2);		// inicia simulacion hasta que salga del laberinto
	route_1.write(maze_out_1);						// escribe ruta en laberinto para robot
	route_2.write(maze_out_2);						// escribe ruta en laberinto para otro robot

	maze_in.close();		// Cierra simple_maze.pbm
	maze_out_1.close();		// Cierra robot_route_1.pbm
	maze_out_2.close();		// Cierra robot_route_2.pbm
	return 0;
}
